<?php

declare(strict_types=1);

namespace Drupal\graphql_compose_edges\Plugin\GraphQLCompose\SchemaType;

use Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeSchemaTypeBase;
use Drupal\graphql_compose_edges\EnabledBundlesTrait;
use GraphQL\Type\Definition\InterfaceType;
use GraphQL\Type\Definition\ObjectType;
use GraphQL\Type\Definition\Type;

/**
 * {@inheritDoc}
 *
 * @GraphQLComposeSchemaType(
 *   id = "Connection",
 * )
 */
class ConnectionType extends GraphQLComposeSchemaTypeBase {

  use EnabledBundlesTrait;

  /**
   * {@inheritDoc}
   */
  public function getTypes(): array {
    $types = [];

    $types[] = new InterfaceType([
      'name' => $this->getPluginId(),
      'description' => (string) $this->t('A paginated set of results.'),
      'fields' => fn() => [
        'edges' => [
          'type' => Type::nonNull(Type::listOf(Type::nonNull(static::type('Edge')))),
          'description' => (string) $this->t('The edges of this connection.'),
        ],
        'nodes' => [
          'type' => Type::nonNull(Type::listOf(Type::nonNull(static::type('Node')))),
          'description' => (string) $this->t('The nodes of the edges of this connection.'),
        ],
        'pageInfo' => [
          'type' => Type::nonNull(static::type('ConnectionPageInfo')),
          'description' => (string) $this->t('Information to aid in pagination.'),
        ],
      ],
    ]);

    foreach ($this->getEnabledBundlePlugins() as $bundle) {
      $type_sdl = $bundle->getTypeSdl();

      $types[] = new ObjectType([
        'name' => $type_sdl . 'Connection',
        'description' => (string) $this->t('A paginated set of results for @bundle.', [
          '@bundle' => $type_sdl,
        ]),
        'interfaces' => fn() => [
          static::type('Connection'),
        ],
        'fields' => fn() => [
          'edges' => Type::nonNull(Type::listOf(Type::nonNull(static::type($type_sdl . 'Edge')))),
          'nodes' => Type::nonNull(Type::listOf(Type::nonNull(static::type($type_sdl)))),
          'pageInfo' => Type::nonNull(static::type('ConnectionPageInfo')),
        ],
      ]);
    }

    return $types;
  }

  /**
   * {@inheritDoc}
   *
   * Create bundle queries.
   */
  public function getExtensions(): array {
    $extensions = parent::getExtensions();

    foreach ($this->getEnabledBundlePlugins() as $bundle) {
      $type_sdl = $bundle->getTypeSdl();

      $extensions[] = new ObjectType([
        'name' => 'Query',
        'fields' => fn() => [
          $bundle->getNamePluralSdl() => [
            'type' => Type::nonNull(static::type($type_sdl . 'Connection')),
            'description' => (string) $this->t('List of all @bundle on the platform.', [
              '@bundle' => $type_sdl,
            ]),
            'args' => [
              'after' => [
                'type' => static::type('Cursor'),
                'description' => (string) $this->t('Returns the elements that come after the specified cursor.'),
              ],
              'before' => [
                'type' => static::type('Cursor'),
                'description' => (string) $this->t('Returns the elements that come before the specified cursor.'),
              ],
              'first' => [
                'type' => Type::int(),
                'description' => (string) $this->t('Returns up to the first n elements from the list.'),
              ],
              'last' => [
                'type' => Type::int(),
                'description' => (string) $this->t('Returns up to the last n elements from the list.'),
              ],
              'reverse' => [
                'type' => Type::boolean(),
                'defaultValue' => FALSE,
                'description' => (string) $this->t('Reverse the order of the underlying list.'),
              ],
              'sortKey' => [
                'type' => static::type('ConnectionSortKeys'),
                'defaultValue' => 'CREATED_AT',
                'description' => (string) $this->t('Sort the underlying list by the given key.'),
              ],
            ],
          ],
        ],
      ]);
    }

    return $extensions;
  }

}
